/*
  SIM PHONEBOOK READER
  
 This sketch demonstrates the phonebook reading
 of a Subscriber Identity Module (SIM), the smartcard used 
 in cellular phones.
 
 The circuit:
 SmartShield (http://www.elettroinnova.it)
 
 This example code is in the public domain.

*/

// include the SPI library:
#include <SPI.h>

//include the Smartcard library
#include <Smartcard.h>

// initialize the library with address of slave select pin
Smartcard smartcard(7);

//convert from hexadecimal to ASCII for display the result 
void HexToAsc(char *c){
	if (((*c&0x0F)>=0)&&((*c&0x0F)<=9))
		*(c+1) = (*c&0x0F)+0x30;
	else
		*(c+1) = (*c&0x0F)+0x37;

	if ((((*c&0xF0)>>4)>=0)&&(((*c&0xF0)>>4)<=9))
		*c = ((*c&0xF0)>>4)+0x30;
	else
		*c = ((*c&0xF0)>>4)+0x37;
}

void setup() {
  // start the serial library:
  Serial.begin(9600);
}

void loop() {
      unsigned char data[32];
      char ascii_data[2];
      unsigned char C[6];
      unsigned char D[29];
      unsigned char SW[2];
      unsigned char counter;
      unsigned char record;
      unsigned char temp;
      unsigned char i;
      unsigned char length;
     
      //reset SIM
      do {
          length = smartcard.PowerOnATR(0,data);
         }
      while(data[0]  != 0x3B);
      
      //SIM SELECT 7F 10
      C[0] = 0xA0;
      C[1] = 0xA4;
      C[2] = 0x00;
      C[3] = 0x00;
      C[4] = 0x02;
      C[5] = 0x00;
      D[0] = 0x7F;
      D[1] = 0x10;
     
      counter = 0;
      do {
          smartcard.ApduCommand(0,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x9F)||((SW[1] != 0x16))));
      
     //SIM SELECT 6F 3A
      C[0] = 0xA0;
      C[1] = 0xA4;
      C[2] = 0x00;
      C[3] = 0x00;
      C[4] = 0x02;
      C[5] = 0x00;
      D[0] = 0x6F;
      D[1] = 0x3A;
     
      counter = 0;
      do {
          smartcard.ApduCommand(0,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x9F)||((SW[1] != 0x0F))));
      
      for (record = 1; record <= 250; record++)
      {
       //SIM READ RECORD
      C[0] = 0xA0;
      C[1] = 0xB2;
      C[2] = record;
      C[3] = 0x04;
      C[4] = 0x00;
      C[5] = 0x1D;
      D[0] = 0x00;
     
      counter = 0;
      do {
          smartcard.ApduCommand(0,0,C,D,SW);
          delay(10);
          counter++;
         }
      while((counter < 10)&&((SW[0] != 0x90)||((SW[1] != 0x00))));
      
      if(record < 10)
        Serial.print("  ");
        else if(record < 100)
          Serial.print(" ");
      
      Serial.print(record);   
      Serial.print(". ");   
      
      //print name
      for(i=0;i<15;i++){
        if(D[i] == 0xFF)
           D[i] = 0x20;
        Serial.write(D[i]);    
      }
            
      //exchange nibbles
      for(i=17;i<29;i++){
        temp = D[i]&0xF0;  
        temp >>= 4;
        D[i] <<= 4;
        D[i] |= temp;
      }      
      Serial.print(" ");
      
      //print phone number
      for(i=17;i<23;i++){
        ascii_data[0] = D[i];
        HexToAsc(ascii_data);
        if(ascii_data[0] == 'F')
          ascii_data[0] = 0x20;
        if(ascii_data[1] == 'F')
          ascii_data[1] = 0x20;
        Serial.print(ascii_data[0]);
        Serial.print(ascii_data[1]);
      }
       Serial.println();      
      }
 
     while(1);
  }

